// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"net/http"

	"github.com/nicholas-fedor/shoutrrr/pkg/services/push/gotify"
	mock "github.com/stretchr/testify/mock"
)

// NewMockHTTPClientManager creates a new instance of MockHTTPClientManager. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockHTTPClientManager(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockHTTPClientManager {
	mock := &MockHTTPClientManager{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockHTTPClientManager is an autogenerated mock type for the HTTPClientManager type
type MockHTTPClientManager struct {
	mock.Mock
}

type MockHTTPClientManager_Expecter struct {
	mock *mock.Mock
}

func (_m *MockHTTPClientManager) EXPECT() *MockHTTPClientManager_Expecter {
	return &MockHTTPClientManager_Expecter{mock: &_m.Mock}
}

// CreateClient provides a mock function for the type MockHTTPClientManager
func (_mock *MockHTTPClientManager) CreateClient(transport *http.Transport) *http.Client {
	ret := _mock.Called(transport)

	if len(ret) == 0 {
		panic("no return value specified for CreateClient")
	}

	var r0 *http.Client
	if returnFunc, ok := ret.Get(0).(func(*http.Transport) *http.Client); ok {
		r0 = returnFunc(transport)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*http.Client)
		}
	}
	return r0
}

// MockHTTPClientManager_CreateClient_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'CreateClient'
type MockHTTPClientManager_CreateClient_Call struct {
	*mock.Call
}

// CreateClient is a helper method to define mock.On call
//   - transport *http.Transport
func (_e *MockHTTPClientManager_Expecter) CreateClient(transport interface{}) *MockHTTPClientManager_CreateClient_Call {
	return &MockHTTPClientManager_CreateClient_Call{Call: _e.mock.On("CreateClient", transport)}
}

func (_c *MockHTTPClientManager_CreateClient_Call) Run(run func(transport *http.Transport)) *MockHTTPClientManager_CreateClient_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *http.Transport
		if args[0] != nil {
			arg0 = args[0].(*http.Transport)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockHTTPClientManager_CreateClient_Call) Return(client *http.Client) *MockHTTPClientManager_CreateClient_Call {
	_c.Call.Return(client)
	return _c
}

func (_c *MockHTTPClientManager_CreateClient_Call) RunAndReturn(run func(transport *http.Transport) *http.Client) *MockHTTPClientManager_CreateClient_Call {
	_c.Call.Return(run)
	return _c
}

// CreateTransport provides a mock function for the type MockHTTPClientManager
func (_mock *MockHTTPClientManager) CreateTransport(config *gotify.Config) *http.Transport {
	ret := _mock.Called(config)

	if len(ret) == 0 {
		panic("no return value specified for CreateTransport")
	}

	var r0 *http.Transport
	if returnFunc, ok := ret.Get(0).(func(*gotify.Config) *http.Transport); ok {
		r0 = returnFunc(config)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*http.Transport)
		}
	}
	return r0
}

// MockHTTPClientManager_CreateTransport_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'CreateTransport'
type MockHTTPClientManager_CreateTransport_Call struct {
	*mock.Call
}

// CreateTransport is a helper method to define mock.On call
//   - config *gotify.Config
func (_e *MockHTTPClientManager_Expecter) CreateTransport(config interface{}) *MockHTTPClientManager_CreateTransport_Call {
	return &MockHTTPClientManager_CreateTransport_Call{Call: _e.mock.On("CreateTransport", config)}
}

func (_c *MockHTTPClientManager_CreateTransport_Call) Run(run func(config *gotify.Config)) *MockHTTPClientManager_CreateTransport_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *gotify.Config
		if args[0] != nil {
			arg0 = args[0].(*gotify.Config)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockHTTPClientManager_CreateTransport_Call) Return(transport *http.Transport) *MockHTTPClientManager_CreateTransport_Call {
	_c.Call.Return(transport)
	return _c
}

func (_c *MockHTTPClientManager_CreateTransport_Call) RunAndReturn(run func(config *gotify.Config) *http.Transport) *MockHTTPClientManager_CreateTransport_Call {
	_c.Call.Return(run)
	return _c
}
