// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/pkg/services/push/gotify"
	mock "github.com/stretchr/testify/mock"
)

// NewMockURLBuilder creates a new instance of MockURLBuilder. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockURLBuilder(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockURLBuilder {
	mock := &MockURLBuilder{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockURLBuilder is an autogenerated mock type for the URLBuilder type
type MockURLBuilder struct {
	mock.Mock
}

type MockURLBuilder_Expecter struct {
	mock *mock.Mock
}

func (_m *MockURLBuilder) EXPECT() *MockURLBuilder_Expecter {
	return &MockURLBuilder_Expecter{mock: &_m.Mock}
}

// BuildURL provides a mock function for the type MockURLBuilder
func (_mock *MockURLBuilder) BuildURL(config *gotify.Config) (string, error) {
	ret := _mock.Called(config)

	if len(ret) == 0 {
		panic("no return value specified for BuildURL")
	}

	var r0 string
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(*gotify.Config) (string, error)); ok {
		return returnFunc(config)
	}
	if returnFunc, ok := ret.Get(0).(func(*gotify.Config) string); ok {
		r0 = returnFunc(config)
	} else {
		r0 = ret.Get(0).(string)
	}
	if returnFunc, ok := ret.Get(1).(func(*gotify.Config) error); ok {
		r1 = returnFunc(config)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockURLBuilder_BuildURL_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'BuildURL'
type MockURLBuilder_BuildURL_Call struct {
	*mock.Call
}

// BuildURL is a helper method to define mock.On call
//   - config *gotify.Config
func (_e *MockURLBuilder_Expecter) BuildURL(config interface{}) *MockURLBuilder_BuildURL_Call {
	return &MockURLBuilder_BuildURL_Call{Call: _e.mock.On("BuildURL", config)}
}

func (_c *MockURLBuilder_BuildURL_Call) Run(run func(config *gotify.Config)) *MockURLBuilder_BuildURL_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *gotify.Config
		if args[0] != nil {
			arg0 = args[0].(*gotify.Config)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockURLBuilder_BuildURL_Call) Return(s string, err error) *MockURLBuilder_BuildURL_Call {
	_c.Call.Return(s, err)
	return _c
}

func (_c *MockURLBuilder_BuildURL_Call) RunAndReturn(run func(config *gotify.Config) (string, error)) *MockURLBuilder_BuildURL_Call {
	_c.Call.Return(run)
	return _c
}
