// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	mock "github.com/stretchr/testify/mock"
)

// NewMockConfigProp creates a new instance of MockConfigProp. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockConfigProp(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockConfigProp {
	mock := &MockConfigProp{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockConfigProp is an autogenerated mock type for the ConfigProp type
type MockConfigProp struct {
	mock.Mock
}

type MockConfigProp_Expecter struct {
	mock *mock.Mock
}

func (_m *MockConfigProp) EXPECT() *MockConfigProp_Expecter {
	return &MockConfigProp_Expecter{mock: &_m.Mock}
}

// GetPropValue provides a mock function for the type MockConfigProp
func (_mock *MockConfigProp) GetPropValue() (string, error) {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for GetPropValue")
	}

	var r0 string
	var r1 error
	if returnFunc, ok := ret.Get(0).(func() (string, error)); ok {
		return returnFunc()
	}
	if returnFunc, ok := ret.Get(0).(func() string); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Get(0).(string)
	}
	if returnFunc, ok := ret.Get(1).(func() error); ok {
		r1 = returnFunc()
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockConfigProp_GetPropValue_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetPropValue'
type MockConfigProp_GetPropValue_Call struct {
	*mock.Call
}

// GetPropValue is a helper method to define mock.On call
func (_e *MockConfigProp_Expecter) GetPropValue() *MockConfigProp_GetPropValue_Call {
	return &MockConfigProp_GetPropValue_Call{Call: _e.mock.On("GetPropValue")}
}

func (_c *MockConfigProp_GetPropValue_Call) Run(run func()) *MockConfigProp_GetPropValue_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockConfigProp_GetPropValue_Call) Return(s string, err error) *MockConfigProp_GetPropValue_Call {
	_c.Call.Return(s, err)
	return _c
}

func (_c *MockConfigProp_GetPropValue_Call) RunAndReturn(run func() (string, error)) *MockConfigProp_GetPropValue_Call {
	_c.Call.Return(run)
	return _c
}

// SetFromProp provides a mock function for the type MockConfigProp
func (_mock *MockConfigProp) SetFromProp(propValue string) error {
	ret := _mock.Called(propValue)

	if len(ret) == 0 {
		panic("no return value specified for SetFromProp")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string) error); ok {
		r0 = returnFunc(propValue)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockConfigProp_SetFromProp_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SetFromProp'
type MockConfigProp_SetFromProp_Call struct {
	*mock.Call
}

// SetFromProp is a helper method to define mock.On call
//   - propValue string
func (_e *MockConfigProp_Expecter) SetFromProp(propValue interface{}) *MockConfigProp_SetFromProp_Call {
	return &MockConfigProp_SetFromProp_Call{Call: _e.mock.On("SetFromProp", propValue)}
}

func (_c *MockConfigProp_SetFromProp_Call) Run(run func(propValue string)) *MockConfigProp_SetFromProp_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockConfigProp_SetFromProp_Call) Return(err error) *MockConfigProp_SetFromProp_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockConfigProp_SetFromProp_Call) RunAndReturn(run func(propValue string) error) *MockConfigProp_SetFromProp_Call {
	_c.Call.Return(run)
	return _c
}
