// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"text/template"

	mock "github.com/stretchr/testify/mock"
)

// NewMockTemplater creates a new instance of MockTemplater. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockTemplater(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockTemplater {
	mock := &MockTemplater{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockTemplater is an autogenerated mock type for the Templater type
type MockTemplater struct {
	mock.Mock
}

type MockTemplater_Expecter struct {
	mock *mock.Mock
}

func (_m *MockTemplater) EXPECT() *MockTemplater_Expecter {
	return &MockTemplater_Expecter{mock: &_m.Mock}
}

// GetTemplate provides a mock function for the type MockTemplater
func (_mock *MockTemplater) GetTemplate(id string) (*template.Template, bool) {
	ret := _mock.Called(id)

	if len(ret) == 0 {
		panic("no return value specified for GetTemplate")
	}

	var r0 *template.Template
	var r1 bool
	if returnFunc, ok := ret.Get(0).(func(string) (*template.Template, bool)); ok {
		return returnFunc(id)
	}
	if returnFunc, ok := ret.Get(0).(func(string) *template.Template); ok {
		r0 = returnFunc(id)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*template.Template)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(string) bool); ok {
		r1 = returnFunc(id)
	} else {
		r1 = ret.Get(1).(bool)
	}
	return r0, r1
}

// MockTemplater_GetTemplate_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetTemplate'
type MockTemplater_GetTemplate_Call struct {
	*mock.Call
}

// GetTemplate is a helper method to define mock.On call
//   - id string
func (_e *MockTemplater_Expecter) GetTemplate(id interface{}) *MockTemplater_GetTemplate_Call {
	return &MockTemplater_GetTemplate_Call{Call: _e.mock.On("GetTemplate", id)}
}

func (_c *MockTemplater_GetTemplate_Call) Run(run func(id string)) *MockTemplater_GetTemplate_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockTemplater_GetTemplate_Call) Return(template1 *template.Template, found bool) *MockTemplater_GetTemplate_Call {
	_c.Call.Return(template1, found)
	return _c
}

func (_c *MockTemplater_GetTemplate_Call) RunAndReturn(run func(id string) (*template.Template, bool)) *MockTemplater_GetTemplate_Call {
	_c.Call.Return(run)
	return _c
}

// SetTemplateFile provides a mock function for the type MockTemplater
func (_mock *MockTemplater) SetTemplateFile(id string, file string) error {
	ret := _mock.Called(id, file)

	if len(ret) == 0 {
		panic("no return value specified for SetTemplateFile")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, string) error); ok {
		r0 = returnFunc(id, file)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockTemplater_SetTemplateFile_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SetTemplateFile'
type MockTemplater_SetTemplateFile_Call struct {
	*mock.Call
}

// SetTemplateFile is a helper method to define mock.On call
//   - id string
//   - file string
func (_e *MockTemplater_Expecter) SetTemplateFile(id interface{}, file interface{}) *MockTemplater_SetTemplateFile_Call {
	return &MockTemplater_SetTemplateFile_Call{Call: _e.mock.On("SetTemplateFile", id, file)}
}

func (_c *MockTemplater_SetTemplateFile_Call) Run(run func(id string, file string)) *MockTemplater_SetTemplateFile_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockTemplater_SetTemplateFile_Call) Return(err error) *MockTemplater_SetTemplateFile_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockTemplater_SetTemplateFile_Call) RunAndReturn(run func(id string, file string) error) *MockTemplater_SetTemplateFile_Call {
	_c.Call.Return(run)
	return _c
}

// SetTemplateString provides a mock function for the type MockTemplater
func (_mock *MockTemplater) SetTemplateString(id string, body string) error {
	ret := _mock.Called(id, body)

	if len(ret) == 0 {
		panic("no return value specified for SetTemplateString")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, string) error); ok {
		r0 = returnFunc(id, body)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockTemplater_SetTemplateString_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SetTemplateString'
type MockTemplater_SetTemplateString_Call struct {
	*mock.Call
}

// SetTemplateString is a helper method to define mock.On call
//   - id string
//   - body string
func (_e *MockTemplater_Expecter) SetTemplateString(id interface{}, body interface{}) *MockTemplater_SetTemplateString_Call {
	return &MockTemplater_SetTemplateString_Call{Call: _e.mock.On("SetTemplateString", id, body)}
}

func (_c *MockTemplater_SetTemplateString_Call) Run(run func(id string, body string)) *MockTemplater_SetTemplateString_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockTemplater_SetTemplateString_Call) Return(err error) *MockTemplater_SetTemplateString_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockTemplater_SetTemplateString_Call) RunAndReturn(run func(id string, body string) error) *MockTemplater_SetTemplateString_Call {
	_c.Call.Return(run)
	return _c
}
