// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"net/http"

	"github.com/nicholas-fedor/shoutrrr/pkg/services/push/gotify"
	mock "github.com/stretchr/testify/mock"
)

// NewMockSender creates a new instance of MockSender. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockSender(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockSender {
	mock := &MockSender{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockSender is an autogenerated mock type for the Sender type
type MockSender struct {
	mock.Mock
}

type MockSender_Expecter struct {
	mock *mock.Mock
}

func (_m *MockSender) EXPECT() *MockSender_Expecter {
	return &MockSender_Expecter{mock: &_m.Mock}
}

// SendRequest provides a mock function for the type MockSender
func (_mock *MockSender) SendRequest(client *http.Client, url string, request *gotify.MessageRequest, headers http.Header) error {
	ret := _mock.Called(client, url, request, headers)

	if len(ret) == 0 {
		panic("no return value specified for SendRequest")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(*http.Client, string, *gotify.MessageRequest, http.Header) error); ok {
		r0 = returnFunc(client, url, request, headers)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockSender_SendRequest_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SendRequest'
type MockSender_SendRequest_Call struct {
	*mock.Call
}

// SendRequest is a helper method to define mock.On call
//   - client *http.Client
//   - url string
//   - request *gotify.MessageRequest
//   - headers http.Header
func (_e *MockSender_Expecter) SendRequest(client interface{}, url interface{}, request interface{}, headers interface{}) *MockSender_SendRequest_Call {
	return &MockSender_SendRequest_Call{Call: _e.mock.On("SendRequest", client, url, request, headers)}
}

func (_c *MockSender_SendRequest_Call) Run(run func(client *http.Client, url string, request *gotify.MessageRequest, headers http.Header)) *MockSender_SendRequest_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *http.Client
		if args[0] != nil {
			arg0 = args[0].(*http.Client)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		var arg2 *gotify.MessageRequest
		if args[2] != nil {
			arg2 = args[2].(*gotify.MessageRequest)
		}
		var arg3 http.Header
		if args[3] != nil {
			arg3 = args[3].(http.Header)
		}
		run(
			arg0,
			arg1,
			arg2,
			arg3,
		)
	})
	return _c
}

func (_c *MockSender_SendRequest_Call) Return(err error) *MockSender_SendRequest_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockSender_SendRequest_Call) RunAndReturn(run func(client *http.Client, url string, request *gotify.MessageRequest, headers http.Header) error) *MockSender_SendRequest_Call {
	_c.Call.Return(run)
	return _c
}
