// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"context"
	"net/http"

	mock "github.com/stretchr/testify/mock"
)

// NewMockRequestPreparer creates a new instance of MockRequestPreparer. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockRequestPreparer(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockRequestPreparer {
	mock := &MockRequestPreparer{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockRequestPreparer is an autogenerated mock type for the RequestPreparer type
type MockRequestPreparer struct {
	mock.Mock
}

type MockRequestPreparer_Expecter struct {
	mock *mock.Mock
}

func (_m *MockRequestPreparer) EXPECT() *MockRequestPreparer_Expecter {
	return &MockRequestPreparer_Expecter{mock: &_m.Mock}
}

// PrepareRequest provides a mock function for the type MockRequestPreparer
func (_mock *MockRequestPreparer) PrepareRequest(ctx context.Context, requestURL string) (*http.Request, error) {
	ret := _mock.Called(ctx, requestURL)

	if len(ret) == 0 {
		panic("no return value specified for PrepareRequest")
	}

	var r0 *http.Request
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, string) (*http.Request, error)); ok {
		return returnFunc(ctx, requestURL)
	}
	if returnFunc, ok := ret.Get(0).(func(context.Context, string) *http.Request); ok {
		r0 = returnFunc(ctx, requestURL)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*http.Request)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(context.Context, string) error); ok {
		r1 = returnFunc(ctx, requestURL)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockRequestPreparer_PrepareRequest_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'PrepareRequest'
type MockRequestPreparer_PrepareRequest_Call struct {
	*mock.Call
}

// PrepareRequest is a helper method to define mock.On call
//   - ctx context.Context
//   - requestURL string
func (_e *MockRequestPreparer_Expecter) PrepareRequest(ctx interface{}, requestURL interface{}) *MockRequestPreparer_PrepareRequest_Call {
	return &MockRequestPreparer_PrepareRequest_Call{Call: _e.mock.On("PrepareRequest", ctx, requestURL)}
}

func (_c *MockRequestPreparer_PrepareRequest_Call) Run(run func(ctx context.Context, requestURL string)) *MockRequestPreparer_PrepareRequest_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockRequestPreparer_PrepareRequest_Call) Return(request *http.Request, err error) *MockRequestPreparer_PrepareRequest_Call {
	_c.Call.Return(request, err)
	return _c
}

func (_c *MockRequestPreparer_PrepareRequest_Call) RunAndReturn(run func(ctx context.Context, requestURL string) (*http.Request, error)) *MockRequestPreparer_PrepareRequest_Call {
	_c.Call.Return(run)
	return _c
}
