// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/pkg/types"
	mock "github.com/stretchr/testify/mock"
)

// NewMockEnummer creates a new instance of MockEnummer. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockEnummer(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockEnummer {
	mock := &MockEnummer{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockEnummer is an autogenerated mock type for the Enummer type
type MockEnummer struct {
	mock.Mock
}

type MockEnummer_Expecter struct {
	mock *mock.Mock
}

func (_m *MockEnummer) EXPECT() *MockEnummer_Expecter {
	return &MockEnummer_Expecter{mock: &_m.Mock}
}

// Enums provides a mock function for the type MockEnummer
func (_mock *MockEnummer) Enums() map[string]types.EnumFormatter {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Enums")
	}

	var r0 map[string]types.EnumFormatter
	if returnFunc, ok := ret.Get(0).(func() map[string]types.EnumFormatter); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(map[string]types.EnumFormatter)
		}
	}
	return r0
}

// MockEnummer_Enums_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Enums'
type MockEnummer_Enums_Call struct {
	*mock.Call
}

// Enums is a helper method to define mock.On call
func (_e *MockEnummer_Expecter) Enums() *MockEnummer_Enums_Call {
	return &MockEnummer_Enums_Call{Call: _e.mock.On("Enums")}
}

func (_c *MockEnummer_Enums_Call) Run(run func()) *MockEnummer_Enums_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockEnummer_Enums_Call) Return(stringToEnumFormatter map[string]types.EnumFormatter) *MockEnummer_Enums_Call {
	_c.Call.Return(stringToEnumFormatter)
	return _c
}

func (_c *MockEnummer_Enums_Call) RunAndReturn(run func() map[string]types.EnumFormatter) *MockEnummer_Enums_Call {
	_c.Call.Return(run)
	return _c
}
