// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"time"

	mock "github.com/stretchr/testify/mock"
)

// NewMockSleeper creates a new instance of MockSleeper. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockSleeper(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockSleeper {
	mock := &MockSleeper{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockSleeper is an autogenerated mock type for the Sleeper type
type MockSleeper struct {
	mock.Mock
}

type MockSleeper_Expecter struct {
	mock *mock.Mock
}

func (_m *MockSleeper) EXPECT() *MockSleeper_Expecter {
	return &MockSleeper_Expecter{mock: &_m.Mock}
}

// Sleep provides a mock function for the type MockSleeper
func (_mock *MockSleeper) Sleep(d time.Duration) {
	_mock.Called(d)
	return
}

// MockSleeper_Sleep_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Sleep'
type MockSleeper_Sleep_Call struct {
	*mock.Call
}

// Sleep is a helper method to define mock.On call
//   - d time.Duration
func (_e *MockSleeper_Expecter) Sleep(d interface{}) *MockSleeper_Sleep_Call {
	return &MockSleeper_Sleep_Call{Call: _e.mock.On("Sleep", d)}
}

func (_c *MockSleeper_Sleep_Call) Run(run func(d time.Duration)) *MockSleeper_Sleep_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 time.Duration
		if args[0] != nil {
			arg0 = args[0].(time.Duration)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockSleeper_Sleep_Call) Return() *MockSleeper_Sleep_Call {
	_c.Call.Return()
	return _c
}

func (_c *MockSleeper_Sleep_Call) RunAndReturn(run func(d time.Duration)) *MockSleeper_Sleep_Call {
	_c.Run(run)
	return _c
}
