// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	mock "github.com/stretchr/testify/mock"
)

// NewMockStdLogger creates a new instance of MockStdLogger. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockStdLogger(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockStdLogger {
	mock := &MockStdLogger{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockStdLogger is an autogenerated mock type for the StdLogger type
type MockStdLogger struct {
	mock.Mock
}

type MockStdLogger_Expecter struct {
	mock *mock.Mock
}

func (_m *MockStdLogger) EXPECT() *MockStdLogger_Expecter {
	return &MockStdLogger_Expecter{mock: &_m.Mock}
}

// Print provides a mock function for the type MockStdLogger
func (_mock *MockStdLogger) Print(args ...any) {
	if len(args) > 0 {
		_mock.Called(args)
	} else {
		_mock.Called()
	}

	return
}

// MockStdLogger_Print_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Print'
type MockStdLogger_Print_Call struct {
	*mock.Call
}

// Print is a helper method to define mock.On call
//   - args ...any
func (_e *MockStdLogger_Expecter) Print(args ...interface{}) *MockStdLogger_Print_Call {
	return &MockStdLogger_Print_Call{Call: _e.mock.On("Print",
		append([]interface{}{}, args...)...)}
}

func (_c *MockStdLogger_Print_Call) Run(run func(args ...any)) *MockStdLogger_Print_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 []any
		var variadicArgs []any
		if len(args) > 0 {
			variadicArgs = args[0].([]any)
		}
		arg0 = variadicArgs
		run(
			arg0...,
		)
	})
	return _c
}

func (_c *MockStdLogger_Print_Call) Return() *MockStdLogger_Print_Call {
	_c.Call.Return()
	return _c
}

func (_c *MockStdLogger_Print_Call) RunAndReturn(run func(args ...any)) *MockStdLogger_Print_Call {
	_c.Run(run)
	return _c
}

// Printf provides a mock function for the type MockStdLogger
func (_mock *MockStdLogger) Printf(format string, args ...any) {
	if len(args) > 0 {
		_mock.Called(format, args)
	} else {
		_mock.Called(format)
	}

	return
}

// MockStdLogger_Printf_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Printf'
type MockStdLogger_Printf_Call struct {
	*mock.Call
}

// Printf is a helper method to define mock.On call
//   - format string
//   - args ...any
func (_e *MockStdLogger_Expecter) Printf(format interface{}, args ...interface{}) *MockStdLogger_Printf_Call {
	return &MockStdLogger_Printf_Call{Call: _e.mock.On("Printf",
		append([]interface{}{format}, args...)...)}
}

func (_c *MockStdLogger_Printf_Call) Run(run func(format string, args ...any)) *MockStdLogger_Printf_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 []any
		var variadicArgs []any
		if len(args) > 1 {
			variadicArgs = args[1].([]any)
		}
		arg1 = variadicArgs
		run(
			arg0,
			arg1...,
		)
	})
	return _c
}

func (_c *MockStdLogger_Printf_Call) Return() *MockStdLogger_Printf_Call {
	_c.Call.Return()
	return _c
}

func (_c *MockStdLogger_Printf_Call) RunAndReturn(run func(format string, args ...any)) *MockStdLogger_Printf_Call {
	_c.Run(run)
	return _c
}

// Println provides a mock function for the type MockStdLogger
func (_mock *MockStdLogger) Println(args ...any) {
	if len(args) > 0 {
		_mock.Called(args)
	} else {
		_mock.Called()
	}

	return
}

// MockStdLogger_Println_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Println'
type MockStdLogger_Println_Call struct {
	*mock.Call
}

// Println is a helper method to define mock.On call
//   - args ...any
func (_e *MockStdLogger_Expecter) Println(args ...interface{}) *MockStdLogger_Println_Call {
	return &MockStdLogger_Println_Call{Call: _e.mock.On("Println",
		append([]interface{}{}, args...)...)}
}

func (_c *MockStdLogger_Println_Call) Run(run func(args ...any)) *MockStdLogger_Println_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 []any
		var variadicArgs []any
		if len(args) > 0 {
			variadicArgs = args[0].([]any)
		}
		arg0 = variadicArgs
		run(
			arg0...,
		)
	})
	return _c
}

func (_c *MockStdLogger_Println_Call) Return() *MockStdLogger_Println_Call {
	_c.Call.Return()
	return _c
}

func (_c *MockStdLogger_Println_Call) RunAndReturn(run func(args ...any)) *MockStdLogger_Println_Call {
	_c.Run(run)
	return _c
}
