// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/pkg/format"
	mock "github.com/stretchr/testify/mock"
)

// NewMockTreeRenderer creates a new instance of MockTreeRenderer. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockTreeRenderer(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockTreeRenderer {
	mock := &MockTreeRenderer{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockTreeRenderer is an autogenerated mock type for the TreeRenderer type
type MockTreeRenderer struct {
	mock.Mock
}

type MockTreeRenderer_Expecter struct {
	mock *mock.Mock
}

func (_m *MockTreeRenderer) EXPECT() *MockTreeRenderer_Expecter {
	return &MockTreeRenderer_Expecter{mock: &_m.Mock}
}

// RenderTree provides a mock function for the type MockTreeRenderer
func (_mock *MockTreeRenderer) RenderTree(root *format.ContainerNode, scheme string) string {
	ret := _mock.Called(root, scheme)

	if len(ret) == 0 {
		panic("no return value specified for RenderTree")
	}

	var r0 string
	if returnFunc, ok := ret.Get(0).(func(*format.ContainerNode, string) string); ok {
		r0 = returnFunc(root, scheme)
	} else {
		r0 = ret.Get(0).(string)
	}
	return r0
}

// MockTreeRenderer_RenderTree_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'RenderTree'
type MockTreeRenderer_RenderTree_Call struct {
	*mock.Call
}

// RenderTree is a helper method to define mock.On call
//   - root *format.ContainerNode
//   - scheme string
func (_e *MockTreeRenderer_Expecter) RenderTree(root interface{}, scheme interface{}) *MockTreeRenderer_RenderTree_Call {
	return &MockTreeRenderer_RenderTree_Call{Call: _e.mock.On("RenderTree", root, scheme)}
}

func (_c *MockTreeRenderer_RenderTree_Call) Run(run func(root *format.ContainerNode, scheme string)) *MockTreeRenderer_RenderTree_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *format.ContainerNode
		if args[0] != nil {
			arg0 = args[0].(*format.ContainerNode)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockTreeRenderer_RenderTree_Call) Return(s string) *MockTreeRenderer_RenderTree_Call {
	_c.Call.Return(s)
	return _c
}

func (_c *MockTreeRenderer_RenderTree_Call) RunAndReturn(run func(root *format.ContainerNode, scheme string) string) *MockTreeRenderer_RenderTree_Call {
	_c.Call.Return(run)
	return _c
}
