// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	mock "github.com/stretchr/testify/mock"
)

// NewMockConfigQueryResolver creates a new instance of MockConfigQueryResolver. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockConfigQueryResolver(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockConfigQueryResolver {
	mock := &MockConfigQueryResolver{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockConfigQueryResolver is an autogenerated mock type for the ConfigQueryResolver type
type MockConfigQueryResolver struct {
	mock.Mock
}

type MockConfigQueryResolver_Expecter struct {
	mock *mock.Mock
}

func (_m *MockConfigQueryResolver) EXPECT() *MockConfigQueryResolver_Expecter {
	return &MockConfigQueryResolver_Expecter{mock: &_m.Mock}
}

// Get provides a mock function for the type MockConfigQueryResolver
func (_mock *MockConfigQueryResolver) Get(key string) (string, error) {
	ret := _mock.Called(key)

	if len(ret) == 0 {
		panic("no return value specified for Get")
	}

	var r0 string
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(string) (string, error)); ok {
		return returnFunc(key)
	}
	if returnFunc, ok := ret.Get(0).(func(string) string); ok {
		r0 = returnFunc(key)
	} else {
		r0 = ret.Get(0).(string)
	}
	if returnFunc, ok := ret.Get(1).(func(string) error); ok {
		r1 = returnFunc(key)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockConfigQueryResolver_Get_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Get'
type MockConfigQueryResolver_Get_Call struct {
	*mock.Call
}

// Get is a helper method to define mock.On call
//   - key string
func (_e *MockConfigQueryResolver_Expecter) Get(key interface{}) *MockConfigQueryResolver_Get_Call {
	return &MockConfigQueryResolver_Get_Call{Call: _e.mock.On("Get", key)}
}

func (_c *MockConfigQueryResolver_Get_Call) Run(run func(key string)) *MockConfigQueryResolver_Get_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockConfigQueryResolver_Get_Call) Return(value string, err error) *MockConfigQueryResolver_Get_Call {
	_c.Call.Return(value, err)
	return _c
}

func (_c *MockConfigQueryResolver_Get_Call) RunAndReturn(run func(key string) (string, error)) *MockConfigQueryResolver_Get_Call {
	_c.Call.Return(run)
	return _c
}

// QueryFields provides a mock function for the type MockConfigQueryResolver
func (_mock *MockConfigQueryResolver) QueryFields() []string {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for QueryFields")
	}

	var r0 []string
	if returnFunc, ok := ret.Get(0).(func() []string); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).([]string)
		}
	}
	return r0
}

// MockConfigQueryResolver_QueryFields_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'QueryFields'
type MockConfigQueryResolver_QueryFields_Call struct {
	*mock.Call
}

// QueryFields is a helper method to define mock.On call
func (_e *MockConfigQueryResolver_Expecter) QueryFields() *MockConfigQueryResolver_QueryFields_Call {
	return &MockConfigQueryResolver_QueryFields_Call{Call: _e.mock.On("QueryFields")}
}

func (_c *MockConfigQueryResolver_QueryFields_Call) Run(run func()) *MockConfigQueryResolver_QueryFields_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockConfigQueryResolver_QueryFields_Call) Return(strings []string) *MockConfigQueryResolver_QueryFields_Call {
	_c.Call.Return(strings)
	return _c
}

func (_c *MockConfigQueryResolver_QueryFields_Call) RunAndReturn(run func() []string) *MockConfigQueryResolver_QueryFields_Call {
	_c.Call.Return(run)
	return _c
}

// Set provides a mock function for the type MockConfigQueryResolver
func (_mock *MockConfigQueryResolver) Set(key string, value string) error {
	ret := _mock.Called(key, value)

	if len(ret) == 0 {
		panic("no return value specified for Set")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, string) error); ok {
		r0 = returnFunc(key, value)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockConfigQueryResolver_Set_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Set'
type MockConfigQueryResolver_Set_Call struct {
	*mock.Call
}

// Set is a helper method to define mock.On call
//   - key string
//   - value string
func (_e *MockConfigQueryResolver_Expecter) Set(key interface{}, value interface{}) *MockConfigQueryResolver_Set_Call {
	return &MockConfigQueryResolver_Set_Call{Call: _e.mock.On("Set", key, value)}
}

func (_c *MockConfigQueryResolver_Set_Call) Run(run func(key string, value string)) *MockConfigQueryResolver_Set_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockConfigQueryResolver_Set_Call) Return(err error) *MockConfigQueryResolver_Set_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockConfigQueryResolver_Set_Call) RunAndReturn(run func(key string, value string) error) *MockConfigQueryResolver_Set_Call {
	_c.Call.Return(run)
	return _c
}
