// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	mock "github.com/stretchr/testify/mock"
)

// NewMockEnumFormatter creates a new instance of MockEnumFormatter. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockEnumFormatter(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockEnumFormatter {
	mock := &MockEnumFormatter{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockEnumFormatter is an autogenerated mock type for the EnumFormatter type
type MockEnumFormatter struct {
	mock.Mock
}

type MockEnumFormatter_Expecter struct {
	mock *mock.Mock
}

func (_m *MockEnumFormatter) EXPECT() *MockEnumFormatter_Expecter {
	return &MockEnumFormatter_Expecter{mock: &_m.Mock}
}

// Names provides a mock function for the type MockEnumFormatter
func (_mock *MockEnumFormatter) Names() []string {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Names")
	}

	var r0 []string
	if returnFunc, ok := ret.Get(0).(func() []string); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).([]string)
		}
	}
	return r0
}

// MockEnumFormatter_Names_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Names'
type MockEnumFormatter_Names_Call struct {
	*mock.Call
}

// Names is a helper method to define mock.On call
func (_e *MockEnumFormatter_Expecter) Names() *MockEnumFormatter_Names_Call {
	return &MockEnumFormatter_Names_Call{Call: _e.mock.On("Names")}
}

func (_c *MockEnumFormatter_Names_Call) Run(run func()) *MockEnumFormatter_Names_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockEnumFormatter_Names_Call) Return(strings []string) *MockEnumFormatter_Names_Call {
	_c.Call.Return(strings)
	return _c
}

func (_c *MockEnumFormatter_Names_Call) RunAndReturn(run func() []string) *MockEnumFormatter_Names_Call {
	_c.Call.Return(run)
	return _c
}

// Parse provides a mock function for the type MockEnumFormatter
func (_mock *MockEnumFormatter) Parse(s string) int {
	ret := _mock.Called(s)

	if len(ret) == 0 {
		panic("no return value specified for Parse")
	}

	var r0 int
	if returnFunc, ok := ret.Get(0).(func(string) int); ok {
		r0 = returnFunc(s)
	} else {
		r0 = ret.Get(0).(int)
	}
	return r0
}

// MockEnumFormatter_Parse_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Parse'
type MockEnumFormatter_Parse_Call struct {
	*mock.Call
}

// Parse is a helper method to define mock.On call
//   - s string
func (_e *MockEnumFormatter_Expecter) Parse(s interface{}) *MockEnumFormatter_Parse_Call {
	return &MockEnumFormatter_Parse_Call{Call: _e.mock.On("Parse", s)}
}

func (_c *MockEnumFormatter_Parse_Call) Run(run func(s string)) *MockEnumFormatter_Parse_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockEnumFormatter_Parse_Call) Return(n int) *MockEnumFormatter_Parse_Call {
	_c.Call.Return(n)
	return _c
}

func (_c *MockEnumFormatter_Parse_Call) RunAndReturn(run func(s string) int) *MockEnumFormatter_Parse_Call {
	_c.Call.Return(run)
	return _c
}

// Print provides a mock function for the type MockEnumFormatter
func (_mock *MockEnumFormatter) Print(e int) string {
	ret := _mock.Called(e)

	if len(ret) == 0 {
		panic("no return value specified for Print")
	}

	var r0 string
	if returnFunc, ok := ret.Get(0).(func(int) string); ok {
		r0 = returnFunc(e)
	} else {
		r0 = ret.Get(0).(string)
	}
	return r0
}

// MockEnumFormatter_Print_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Print'
type MockEnumFormatter_Print_Call struct {
	*mock.Call
}

// Print is a helper method to define mock.On call
//   - e int
func (_e *MockEnumFormatter_Expecter) Print(e interface{}) *MockEnumFormatter_Print_Call {
	return &MockEnumFormatter_Print_Call{Call: _e.mock.On("Print", e)}
}

func (_c *MockEnumFormatter_Print_Call) Run(run func(e int)) *MockEnumFormatter_Print_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 int
		if args[0] != nil {
			arg0 = args[0].(int)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockEnumFormatter_Print_Call) Return(s string) *MockEnumFormatter_Print_Call {
	_c.Call.Return(s)
	return _c
}

func (_c *MockEnumFormatter_Print_Call) RunAndReturn(run func(e int) string) *MockEnumFormatter_Print_Call {
	_c.Call.Return(run)
	return _c
}
