// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"net/url"
	"text/template"

	"github.com/nicholas-fedor/shoutrrr/pkg/types"
	mock "github.com/stretchr/testify/mock"
)

// NewMockCustomURLService creates a new instance of MockCustomURLService. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockCustomURLService(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockCustomURLService {
	mock := &MockCustomURLService{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockCustomURLService is an autogenerated mock type for the CustomURLService type
type MockCustomURLService struct {
	mock.Mock
}

type MockCustomURLService_Expecter struct {
	mock *mock.Mock
}

func (_m *MockCustomURLService) EXPECT() *MockCustomURLService_Expecter {
	return &MockCustomURLService_Expecter{mock: &_m.Mock}
}

// GetID provides a mock function for the type MockCustomURLService
func (_mock *MockCustomURLService) GetID() string {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for GetID")
	}

	var r0 string
	if returnFunc, ok := ret.Get(0).(func() string); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Get(0).(string)
	}
	return r0
}

// MockCustomURLService_GetID_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetID'
type MockCustomURLService_GetID_Call struct {
	*mock.Call
}

// GetID is a helper method to define mock.On call
func (_e *MockCustomURLService_Expecter) GetID() *MockCustomURLService_GetID_Call {
	return &MockCustomURLService_GetID_Call{Call: _e.mock.On("GetID")}
}

func (_c *MockCustomURLService_GetID_Call) Run(run func()) *MockCustomURLService_GetID_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockCustomURLService_GetID_Call) Return(s string) *MockCustomURLService_GetID_Call {
	_c.Call.Return(s)
	return _c
}

func (_c *MockCustomURLService_GetID_Call) RunAndReturn(run func() string) *MockCustomURLService_GetID_Call {
	_c.Call.Return(run)
	return _c
}

// GetServiceURLFromCustom provides a mock function for the type MockCustomURLService
func (_mock *MockCustomURLService) GetServiceURLFromCustom(customURL *url.URL) (*url.URL, error) {
	ret := _mock.Called(customURL)

	if len(ret) == 0 {
		panic("no return value specified for GetServiceURLFromCustom")
	}

	var r0 *url.URL
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(*url.URL) (*url.URL, error)); ok {
		return returnFunc(customURL)
	}
	if returnFunc, ok := ret.Get(0).(func(*url.URL) *url.URL); ok {
		r0 = returnFunc(customURL)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*url.URL)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(*url.URL) error); ok {
		r1 = returnFunc(customURL)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockCustomURLService_GetServiceURLFromCustom_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetServiceURLFromCustom'
type MockCustomURLService_GetServiceURLFromCustom_Call struct {
	*mock.Call
}

// GetServiceURLFromCustom is a helper method to define mock.On call
//   - customURL *url.URL
func (_e *MockCustomURLService_Expecter) GetServiceURLFromCustom(customURL interface{}) *MockCustomURLService_GetServiceURLFromCustom_Call {
	return &MockCustomURLService_GetServiceURLFromCustom_Call{Call: _e.mock.On("GetServiceURLFromCustom", customURL)}
}

func (_c *MockCustomURLService_GetServiceURLFromCustom_Call) Run(run func(customURL *url.URL)) *MockCustomURLService_GetServiceURLFromCustom_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *url.URL
		if args[0] != nil {
			arg0 = args[0].(*url.URL)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockCustomURLService_GetServiceURLFromCustom_Call) Return(serviceURL *url.URL, err error) *MockCustomURLService_GetServiceURLFromCustom_Call {
	_c.Call.Return(serviceURL, err)
	return _c
}

func (_c *MockCustomURLService_GetServiceURLFromCustom_Call) RunAndReturn(run func(customURL *url.URL) (*url.URL, error)) *MockCustomURLService_GetServiceURLFromCustom_Call {
	_c.Call.Return(run)
	return _c
}

// GetTemplate provides a mock function for the type MockCustomURLService
func (_mock *MockCustomURLService) GetTemplate(id string) (*template.Template, bool) {
	ret := _mock.Called(id)

	if len(ret) == 0 {
		panic("no return value specified for GetTemplate")
	}

	var r0 *template.Template
	var r1 bool
	if returnFunc, ok := ret.Get(0).(func(string) (*template.Template, bool)); ok {
		return returnFunc(id)
	}
	if returnFunc, ok := ret.Get(0).(func(string) *template.Template); ok {
		r0 = returnFunc(id)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*template.Template)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(string) bool); ok {
		r1 = returnFunc(id)
	} else {
		r1 = ret.Get(1).(bool)
	}
	return r0, r1
}

// MockCustomURLService_GetTemplate_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetTemplate'
type MockCustomURLService_GetTemplate_Call struct {
	*mock.Call
}

// GetTemplate is a helper method to define mock.On call
//   - id string
func (_e *MockCustomURLService_Expecter) GetTemplate(id interface{}) *MockCustomURLService_GetTemplate_Call {
	return &MockCustomURLService_GetTemplate_Call{Call: _e.mock.On("GetTemplate", id)}
}

func (_c *MockCustomURLService_GetTemplate_Call) Run(run func(id string)) *MockCustomURLService_GetTemplate_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockCustomURLService_GetTemplate_Call) Return(template1 *template.Template, found bool) *MockCustomURLService_GetTemplate_Call {
	_c.Call.Return(template1, found)
	return _c
}

func (_c *MockCustomURLService_GetTemplate_Call) RunAndReturn(run func(id string) (*template.Template, bool)) *MockCustomURLService_GetTemplate_Call {
	_c.Call.Return(run)
	return _c
}

// Initialize provides a mock function for the type MockCustomURLService
func (_mock *MockCustomURLService) Initialize(serviceURL *url.URL, logger types.StdLogger) error {
	ret := _mock.Called(serviceURL, logger)

	if len(ret) == 0 {
		panic("no return value specified for Initialize")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(*url.URL, types.StdLogger) error); ok {
		r0 = returnFunc(serviceURL, logger)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockCustomURLService_Initialize_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Initialize'
type MockCustomURLService_Initialize_Call struct {
	*mock.Call
}

// Initialize is a helper method to define mock.On call
//   - serviceURL *url.URL
//   - logger types.StdLogger
func (_e *MockCustomURLService_Expecter) Initialize(serviceURL interface{}, logger interface{}) *MockCustomURLService_Initialize_Call {
	return &MockCustomURLService_Initialize_Call{Call: _e.mock.On("Initialize", serviceURL, logger)}
}

func (_c *MockCustomURLService_Initialize_Call) Run(run func(serviceURL *url.URL, logger types.StdLogger)) *MockCustomURLService_Initialize_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *url.URL
		if args[0] != nil {
			arg0 = args[0].(*url.URL)
		}
		var arg1 types.StdLogger
		if args[1] != nil {
			arg1 = args[1].(types.StdLogger)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockCustomURLService_Initialize_Call) Return(err error) *MockCustomURLService_Initialize_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockCustomURLService_Initialize_Call) RunAndReturn(run func(serviceURL *url.URL, logger types.StdLogger) error) *MockCustomURLService_Initialize_Call {
	_c.Call.Return(run)
	return _c
}

// Send provides a mock function for the type MockCustomURLService
func (_mock *MockCustomURLService) Send(message string, params *types.Params) error {
	ret := _mock.Called(message, params)

	if len(ret) == 0 {
		panic("no return value specified for Send")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, *types.Params) error); ok {
		r0 = returnFunc(message, params)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockCustomURLService_Send_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Send'
type MockCustomURLService_Send_Call struct {
	*mock.Call
}

// Send is a helper method to define mock.On call
//   - message string
//   - params *types.Params
func (_e *MockCustomURLService_Expecter) Send(message interface{}, params interface{}) *MockCustomURLService_Send_Call {
	return &MockCustomURLService_Send_Call{Call: _e.mock.On("Send", message, params)}
}

func (_c *MockCustomURLService_Send_Call) Run(run func(message string, params *types.Params)) *MockCustomURLService_Send_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 *types.Params
		if args[1] != nil {
			arg1 = args[1].(*types.Params)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockCustomURLService_Send_Call) Return(err error) *MockCustomURLService_Send_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockCustomURLService_Send_Call) RunAndReturn(run func(message string, params *types.Params) error) *MockCustomURLService_Send_Call {
	_c.Call.Return(run)
	return _c
}

// SetLogger provides a mock function for the type MockCustomURLService
func (_mock *MockCustomURLService) SetLogger(logger types.StdLogger) {
	_mock.Called(logger)
	return
}

// MockCustomURLService_SetLogger_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SetLogger'
type MockCustomURLService_SetLogger_Call struct {
	*mock.Call
}

// SetLogger is a helper method to define mock.On call
//   - logger types.StdLogger
func (_e *MockCustomURLService_Expecter) SetLogger(logger interface{}) *MockCustomURLService_SetLogger_Call {
	return &MockCustomURLService_SetLogger_Call{Call: _e.mock.On("SetLogger", logger)}
}

func (_c *MockCustomURLService_SetLogger_Call) Run(run func(logger types.StdLogger)) *MockCustomURLService_SetLogger_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 types.StdLogger
		if args[0] != nil {
			arg0 = args[0].(types.StdLogger)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockCustomURLService_SetLogger_Call) Return() *MockCustomURLService_SetLogger_Call {
	_c.Call.Return()
	return _c
}

func (_c *MockCustomURLService_SetLogger_Call) RunAndReturn(run func(logger types.StdLogger)) *MockCustomURLService_SetLogger_Call {
	_c.Run(run)
	return _c
}

// SetTemplateFile provides a mock function for the type MockCustomURLService
func (_mock *MockCustomURLService) SetTemplateFile(id string, file string) error {
	ret := _mock.Called(id, file)

	if len(ret) == 0 {
		panic("no return value specified for SetTemplateFile")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, string) error); ok {
		r0 = returnFunc(id, file)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockCustomURLService_SetTemplateFile_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SetTemplateFile'
type MockCustomURLService_SetTemplateFile_Call struct {
	*mock.Call
}

// SetTemplateFile is a helper method to define mock.On call
//   - id string
//   - file string
func (_e *MockCustomURLService_Expecter) SetTemplateFile(id interface{}, file interface{}) *MockCustomURLService_SetTemplateFile_Call {
	return &MockCustomURLService_SetTemplateFile_Call{Call: _e.mock.On("SetTemplateFile", id, file)}
}

func (_c *MockCustomURLService_SetTemplateFile_Call) Run(run func(id string, file string)) *MockCustomURLService_SetTemplateFile_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockCustomURLService_SetTemplateFile_Call) Return(err error) *MockCustomURLService_SetTemplateFile_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockCustomURLService_SetTemplateFile_Call) RunAndReturn(run func(id string, file string) error) *MockCustomURLService_SetTemplateFile_Call {
	_c.Call.Return(run)
	return _c
}

// SetTemplateString provides a mock function for the type MockCustomURLService
func (_mock *MockCustomURLService) SetTemplateString(id string, body string) error {
	ret := _mock.Called(id, body)

	if len(ret) == 0 {
		panic("no return value specified for SetTemplateString")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, string) error); ok {
		r0 = returnFunc(id, body)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockCustomURLService_SetTemplateString_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SetTemplateString'
type MockCustomURLService_SetTemplateString_Call struct {
	*mock.Call
}

// SetTemplateString is a helper method to define mock.On call
//   - id string
//   - body string
func (_e *MockCustomURLService_Expecter) SetTemplateString(id interface{}, body interface{}) *MockCustomURLService_SetTemplateString_Call {
	return &MockCustomURLService_SetTemplateString_Call{Call: _e.mock.On("SetTemplateString", id, body)}
}

func (_c *MockCustomURLService_SetTemplateString_Call) Run(run func(id string, body string)) *MockCustomURLService_SetTemplateString_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockCustomURLService_SetTemplateString_Call) Return(err error) *MockCustomURLService_SetTemplateString_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockCustomURLService_SetTemplateString_Call) RunAndReturn(run func(id string, body string) error) *MockCustomURLService_SetTemplateString_Call {
	_c.Call.Return(run)
	return _c
}
