// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/pkg/types"
	mock "github.com/stretchr/testify/mock"
)

// NewMockRichSender creates a new instance of MockRichSender. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockRichSender(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockRichSender {
	mock := &MockRichSender{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockRichSender is an autogenerated mock type for the RichSender type
type MockRichSender struct {
	mock.Mock
}

type MockRichSender_Expecter struct {
	mock *mock.Mock
}

func (_m *MockRichSender) EXPECT() *MockRichSender_Expecter {
	return &MockRichSender_Expecter{mock: &_m.Mock}
}

// SendItems provides a mock function for the type MockRichSender
func (_mock *MockRichSender) SendItems(items []types.MessageItem, params types.Params) error {
	ret := _mock.Called(items, params)

	if len(ret) == 0 {
		panic("no return value specified for SendItems")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func([]types.MessageItem, types.Params) error); ok {
		r0 = returnFunc(items, params)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockRichSender_SendItems_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SendItems'
type MockRichSender_SendItems_Call struct {
	*mock.Call
}

// SendItems is a helper method to define mock.On call
//   - items []types.MessageItem
//   - params types.Params
func (_e *MockRichSender_Expecter) SendItems(items interface{}, params interface{}) *MockRichSender_SendItems_Call {
	return &MockRichSender_SendItems_Call{Call: _e.mock.On("SendItems", items, params)}
}

func (_c *MockRichSender_SendItems_Call) Run(run func(items []types.MessageItem, params types.Params)) *MockRichSender_SendItems_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 []types.MessageItem
		if args[0] != nil {
			arg0 = args[0].([]types.MessageItem)
		}
		var arg1 types.Params
		if args[1] != nil {
			arg1 = args[1].(types.Params)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockRichSender_SendItems_Call) Return(err error) *MockRichSender_SendItems_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockRichSender_SendItems_Call) RunAndReturn(run func(items []types.MessageItem, params types.Params) error) *MockRichSender_SendItems_Call {
	_c.Call.Return(run)
	return _c
}
