// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/pkg/services/push/gotify"
	mock "github.com/stretchr/testify/mock"
)

// NewMockValidator creates a new instance of MockValidator. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockValidator(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockValidator {
	mock := &MockValidator{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockValidator is an autogenerated mock type for the Validator type
type MockValidator struct {
	mock.Mock
}

type MockValidator_Expecter struct {
	mock *mock.Mock
}

func (_m *MockValidator) EXPECT() *MockValidator_Expecter {
	return &MockValidator_Expecter{mock: &_m.Mock}
}

// ValidateDate provides a mock function for the type MockValidator
func (_mock *MockValidator) ValidateDate(date string) (string, error) {
	ret := _mock.Called(date)

	if len(ret) == 0 {
		panic("no return value specified for ValidateDate")
	}

	var r0 string
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(string) (string, error)); ok {
		return returnFunc(date)
	}
	if returnFunc, ok := ret.Get(0).(func(string) string); ok {
		r0 = returnFunc(date)
	} else {
		r0 = ret.Get(0).(string)
	}
	if returnFunc, ok := ret.Get(1).(func(string) error); ok {
		r1 = returnFunc(date)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockValidator_ValidateDate_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'ValidateDate'
type MockValidator_ValidateDate_Call struct {
	*mock.Call
}

// ValidateDate is a helper method to define mock.On call
//   - date string
func (_e *MockValidator_Expecter) ValidateDate(date interface{}) *MockValidator_ValidateDate_Call {
	return &MockValidator_ValidateDate_Call{Call: _e.mock.On("ValidateDate", date)}
}

func (_c *MockValidator_ValidateDate_Call) Run(run func(date string)) *MockValidator_ValidateDate_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockValidator_ValidateDate_Call) Return(s string, err error) *MockValidator_ValidateDate_Call {
	_c.Call.Return(s, err)
	return _c
}

func (_c *MockValidator_ValidateDate_Call) RunAndReturn(run func(date string) (string, error)) *MockValidator_ValidateDate_Call {
	_c.Call.Return(run)
	return _c
}

// ValidateMessage provides a mock function for the type MockValidator
func (_mock *MockValidator) ValidateMessage(message string) error {
	ret := _mock.Called(message)

	if len(ret) == 0 {
		panic("no return value specified for ValidateMessage")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string) error); ok {
		r0 = returnFunc(message)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockValidator_ValidateMessage_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'ValidateMessage'
type MockValidator_ValidateMessage_Call struct {
	*mock.Call
}

// ValidateMessage is a helper method to define mock.On call
//   - message string
func (_e *MockValidator_Expecter) ValidateMessage(message interface{}) *MockValidator_ValidateMessage_Call {
	return &MockValidator_ValidateMessage_Call{Call: _e.mock.On("ValidateMessage", message)}
}

func (_c *MockValidator_ValidateMessage_Call) Run(run func(message string)) *MockValidator_ValidateMessage_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockValidator_ValidateMessage_Call) Return(err error) *MockValidator_ValidateMessage_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockValidator_ValidateMessage_Call) RunAndReturn(run func(message string) error) *MockValidator_ValidateMessage_Call {
	_c.Call.Return(run)
	return _c
}

// ValidatePriority provides a mock function for the type MockValidator
func (_mock *MockValidator) ValidatePriority(priority int) error {
	ret := _mock.Called(priority)

	if len(ret) == 0 {
		panic("no return value specified for ValidatePriority")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(int) error); ok {
		r0 = returnFunc(priority)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockValidator_ValidatePriority_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'ValidatePriority'
type MockValidator_ValidatePriority_Call struct {
	*mock.Call
}

// ValidatePriority is a helper method to define mock.On call
//   - priority int
func (_e *MockValidator_Expecter) ValidatePriority(priority interface{}) *MockValidator_ValidatePriority_Call {
	return &MockValidator_ValidatePriority_Call{Call: _e.mock.On("ValidatePriority", priority)}
}

func (_c *MockValidator_ValidatePriority_Call) Run(run func(priority int)) *MockValidator_ValidatePriority_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 int
		if args[0] != nil {
			arg0 = args[0].(int)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockValidator_ValidatePriority_Call) Return(err error) *MockValidator_ValidatePriority_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockValidator_ValidatePriority_Call) RunAndReturn(run func(priority int) error) *MockValidator_ValidatePriority_Call {
	_c.Call.Return(run)
	return _c
}

// ValidateServiceInitialized provides a mock function for the type MockValidator
func (_mock *MockValidator) ValidateServiceInitialized(config *gotify.Config) error {
	ret := _mock.Called(config)

	if len(ret) == 0 {
		panic("no return value specified for ValidateServiceInitialized")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(*gotify.Config) error); ok {
		r0 = returnFunc(config)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockValidator_ValidateServiceInitialized_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'ValidateServiceInitialized'
type MockValidator_ValidateServiceInitialized_Call struct {
	*mock.Call
}

// ValidateServiceInitialized is a helper method to define mock.On call
//   - config *gotify.Config
func (_e *MockValidator_Expecter) ValidateServiceInitialized(config interface{}) *MockValidator_ValidateServiceInitialized_Call {
	return &MockValidator_ValidateServiceInitialized_Call{Call: _e.mock.On("ValidateServiceInitialized", config)}
}

func (_c *MockValidator_ValidateServiceInitialized_Call) Run(run func(config *gotify.Config)) *MockValidator_ValidateServiceInitialized_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *gotify.Config
		if args[0] != nil {
			arg0 = args[0].(*gotify.Config)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockValidator_ValidateServiceInitialized_Call) Return(err error) *MockValidator_ValidateServiceInitialized_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockValidator_ValidateServiceInitialized_Call) RunAndReturn(run func(config *gotify.Config) error) *MockValidator_ValidateServiceInitialized_Call {
	_c.Call.Return(run)
	return _c
}

// ValidateToken provides a mock function for the type MockValidator
func (_mock *MockValidator) ValidateToken(token string) bool {
	ret := _mock.Called(token)

	if len(ret) == 0 {
		panic("no return value specified for ValidateToken")
	}

	var r0 bool
	if returnFunc, ok := ret.Get(0).(func(string) bool); ok {
		r0 = returnFunc(token)
	} else {
		r0 = ret.Get(0).(bool)
	}
	return r0
}

// MockValidator_ValidateToken_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'ValidateToken'
type MockValidator_ValidateToken_Call struct {
	*mock.Call
}

// ValidateToken is a helper method to define mock.On call
//   - token string
func (_e *MockValidator_Expecter) ValidateToken(token interface{}) *MockValidator_ValidateToken_Call {
	return &MockValidator_ValidateToken_Call{Call: _e.mock.On("ValidateToken", token)}
}

func (_c *MockValidator_ValidateToken_Call) Run(run func(token string)) *MockValidator_ValidateToken_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockValidator_ValidateToken_Call) Return(b bool) *MockValidator_ValidateToken_Call {
	_c.Call.Return(b)
	return _c
}

func (_c *MockValidator_ValidateToken_Call) RunAndReturn(run func(token string) bool) *MockValidator_ValidateToken_Call {
	_c.Call.Return(run)
	return _c
}
