// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/pkg/types"
	mock "github.com/stretchr/testify/mock"
)

// NewMockQueuedSender creates a new instance of MockQueuedSender. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockQueuedSender(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockQueuedSender {
	mock := &MockQueuedSender{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockQueuedSender is an autogenerated mock type for the QueuedSender type
type MockQueuedSender struct {
	mock.Mock
}

type MockQueuedSender_Expecter struct {
	mock *mock.Mock
}

func (_m *MockQueuedSender) EXPECT() *MockQueuedSender_Expecter {
	return &MockQueuedSender_Expecter{mock: &_m.Mock}
}

// Enqueue provides a mock function for the type MockQueuedSender
func (_mock *MockQueuedSender) Enqueue(message string) {
	_mock.Called(message)
	return
}

// MockQueuedSender_Enqueue_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Enqueue'
type MockQueuedSender_Enqueue_Call struct {
	*mock.Call
}

// Enqueue is a helper method to define mock.On call
//   - message string
func (_e *MockQueuedSender_Expecter) Enqueue(message interface{}) *MockQueuedSender_Enqueue_Call {
	return &MockQueuedSender_Enqueue_Call{Call: _e.mock.On("Enqueue", message)}
}

func (_c *MockQueuedSender_Enqueue_Call) Run(run func(message string)) *MockQueuedSender_Enqueue_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockQueuedSender_Enqueue_Call) Return() *MockQueuedSender_Enqueue_Call {
	_c.Call.Return()
	return _c
}

func (_c *MockQueuedSender_Enqueue_Call) RunAndReturn(run func(message string)) *MockQueuedSender_Enqueue_Call {
	_c.Run(run)
	return _c
}

// Enqueuef provides a mock function for the type MockQueuedSender
func (_mock *MockQueuedSender) Enqueuef(format string, v ...any) {
	if len(v) > 0 {
		_mock.Called(format, v)
	} else {
		_mock.Called(format)
	}

	return
}

// MockQueuedSender_Enqueuef_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Enqueuef'
type MockQueuedSender_Enqueuef_Call struct {
	*mock.Call
}

// Enqueuef is a helper method to define mock.On call
//   - format string
//   - v ...any
func (_e *MockQueuedSender_Expecter) Enqueuef(format interface{}, v ...interface{}) *MockQueuedSender_Enqueuef_Call {
	return &MockQueuedSender_Enqueuef_Call{Call: _e.mock.On("Enqueuef",
		append([]interface{}{format}, v...)...)}
}

func (_c *MockQueuedSender_Enqueuef_Call) Run(run func(format string, v ...any)) *MockQueuedSender_Enqueuef_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 []any
		var variadicArgs []any
		if len(args) > 1 {
			variadicArgs = args[1].([]any)
		}
		arg1 = variadicArgs
		run(
			arg0,
			arg1...,
		)
	})
	return _c
}

func (_c *MockQueuedSender_Enqueuef_Call) Return() *MockQueuedSender_Enqueuef_Call {
	_c.Call.Return()
	return _c
}

func (_c *MockQueuedSender_Enqueuef_Call) RunAndReturn(run func(format string, v ...any)) *MockQueuedSender_Enqueuef_Call {
	_c.Run(run)
	return _c
}

// Flush provides a mock function for the type MockQueuedSender
func (_mock *MockQueuedSender) Flush(params *map[string]string) {
	_mock.Called(params)
	return
}

// MockQueuedSender_Flush_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Flush'
type MockQueuedSender_Flush_Call struct {
	*mock.Call
}

// Flush is a helper method to define mock.On call
//   - params *map[string]string
func (_e *MockQueuedSender_Expecter) Flush(params interface{}) *MockQueuedSender_Flush_Call {
	return &MockQueuedSender_Flush_Call{Call: _e.mock.On("Flush", params)}
}

func (_c *MockQueuedSender_Flush_Call) Run(run func(params *map[string]string)) *MockQueuedSender_Flush_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *map[string]string
		if args[0] != nil {
			arg0 = args[0].(*map[string]string)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockQueuedSender_Flush_Call) Return() *MockQueuedSender_Flush_Call {
	_c.Call.Return()
	return _c
}

func (_c *MockQueuedSender_Flush_Call) RunAndReturn(run func(params *map[string]string)) *MockQueuedSender_Flush_Call {
	_c.Run(run)
	return _c
}

// Service provides a mock function for the type MockQueuedSender
func (_mock *MockQueuedSender) Service() types.Service {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Service")
	}

	var r0 types.Service
	if returnFunc, ok := ret.Get(0).(func() types.Service); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(types.Service)
		}
	}
	return r0
}

// MockQueuedSender_Service_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Service'
type MockQueuedSender_Service_Call struct {
	*mock.Call
}

// Service is a helper method to define mock.On call
func (_e *MockQueuedSender_Expecter) Service() *MockQueuedSender_Service_Call {
	return &MockQueuedSender_Service_Call{Call: _e.mock.On("Service")}
}

func (_c *MockQueuedSender_Service_Call) Run(run func()) *MockQueuedSender_Service_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockQueuedSender_Service_Call) Return(service types.Service) *MockQueuedSender_Service_Call {
	_c.Call.Return(service)
	return _c
}

func (_c *MockQueuedSender_Service_Call) RunAndReturn(run func() types.Service) *MockQueuedSender_Service_Call {
	_c.Call.Return(run)
	return _c
}
