// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/pkg/types"
	mock "github.com/stretchr/testify/mock"
)

// NewMockGenerator creates a new instance of MockGenerator. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockGenerator(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockGenerator {
	mock := &MockGenerator{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockGenerator is an autogenerated mock type for the Generator type
type MockGenerator struct {
	mock.Mock
}

type MockGenerator_Expecter struct {
	mock *mock.Mock
}

func (_m *MockGenerator) EXPECT() *MockGenerator_Expecter {
	return &MockGenerator_Expecter{mock: &_m.Mock}
}

// Generate provides a mock function for the type MockGenerator
func (_mock *MockGenerator) Generate(service types.Service, props map[string]string, args []string) (types.ServiceConfig, error) {
	ret := _mock.Called(service, props, args)

	if len(ret) == 0 {
		panic("no return value specified for Generate")
	}

	var r0 types.ServiceConfig
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(types.Service, map[string]string, []string) (types.ServiceConfig, error)); ok {
		return returnFunc(service, props, args)
	}
	if returnFunc, ok := ret.Get(0).(func(types.Service, map[string]string, []string) types.ServiceConfig); ok {
		r0 = returnFunc(service, props, args)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(types.ServiceConfig)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(types.Service, map[string]string, []string) error); ok {
		r1 = returnFunc(service, props, args)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockGenerator_Generate_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Generate'
type MockGenerator_Generate_Call struct {
	*mock.Call
}

// Generate is a helper method to define mock.On call
//   - service types.Service
//   - props map[string]string
//   - args []string
func (_e *MockGenerator_Expecter) Generate(service interface{}, props interface{}, args interface{}) *MockGenerator_Generate_Call {
	return &MockGenerator_Generate_Call{Call: _e.mock.On("Generate", service, props, args)}
}

func (_c *MockGenerator_Generate_Call) Run(run func(service types.Service, props map[string]string, args []string)) *MockGenerator_Generate_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 types.Service
		if args[0] != nil {
			arg0 = args[0].(types.Service)
		}
		var arg1 map[string]string
		if args[1] != nil {
			arg1 = args[1].(map[string]string)
		}
		var arg2 []string
		if args[2] != nil {
			arg2 = args[2].([]string)
		}
		run(
			arg0,
			arg1,
			arg2,
		)
	})
	return _c
}

func (_c *MockGenerator_Generate_Call) Return(serviceConfig types.ServiceConfig, err error) *MockGenerator_Generate_Call {
	_c.Call.Return(serviceConfig, err)
	return _c
}

func (_c *MockGenerator_Generate_Call) RunAndReturn(run func(service types.Service, props map[string]string, args []string) (types.ServiceConfig, error)) *MockGenerator_Generate_Call {
	_c.Call.Return(run)
	return _c
}
