// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/internal/failures"
	mock "github.com/stretchr/testify/mock"
)

// newMockfailure creates a new instance of mockfailure. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func newMockfailure(t interface {
	mock.TestingT
	Cleanup(func())
}) *mockfailure {
	mock := &mockfailure{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// mockfailure is an autogenerated mock type for the failure type
type mockfailure struct {
	mock.Mock
}

type mockfailure_Expecter struct {
	mock *mock.Mock
}

func (_m *mockfailure) EXPECT() *mockfailure_Expecter {
	return &mockfailure_Expecter{mock: &_m.Mock}
}

// Error provides a mock function for the type mockfailure
func (_mock *mockfailure) Error() string {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Error")
	}

	var r0 string
	if returnFunc, ok := ret.Get(0).(func() string); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Get(0).(string)
	}
	return r0
}

// mockfailure_Error_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Error'
type mockfailure_Error_Call struct {
	*mock.Call
}

// Error is a helper method to define mock.On call
func (_e *mockfailure_Expecter) Error() *mockfailure_Error_Call {
	return &mockfailure_Error_Call{Call: _e.mock.On("Error")}
}

func (_c *mockfailure_Error_Call) Run(run func()) *mockfailure_Error_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *mockfailure_Error_Call) Return(s string) *mockfailure_Error_Call {
	_c.Call.Return(s)
	return _c
}

func (_c *mockfailure_Error_Call) RunAndReturn(run func() string) *mockfailure_Error_Call {
	_c.Call.Return(run)
	return _c
}

// ID provides a mock function for the type mockfailure
func (_mock *mockfailure) ID() failures.FailureID {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for ID")
	}

	var r0 failures.FailureID
	if returnFunc, ok := ret.Get(0).(func() failures.FailureID); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Get(0).(failures.FailureID)
	}
	return r0
}

// mockfailure_ID_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'ID'
type mockfailure_ID_Call struct {
	*mock.Call
}

// ID is a helper method to define mock.On call
func (_e *mockfailure_Expecter) ID() *mockfailure_ID_Call {
	return &mockfailure_ID_Call{Call: _e.mock.On("ID")}
}

func (_c *mockfailure_ID_Call) Run(run func()) *mockfailure_ID_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *mockfailure_ID_Call) Return(failureID failures.FailureID) *mockfailure_ID_Call {
	_c.Call.Return(failureID)
	return _c
}

func (_c *mockfailure_ID_Call) RunAndReturn(run func() failures.FailureID) *mockfailure_ID_Call {
	_c.Call.Return(run)
	return _c
}

// Is provides a mock function for the type mockfailure
func (_mock *mockfailure) Is(target error) bool {
	ret := _mock.Called(target)

	if len(ret) == 0 {
		panic("no return value specified for Is")
	}

	var r0 bool
	if returnFunc, ok := ret.Get(0).(func(error) bool); ok {
		r0 = returnFunc(target)
	} else {
		r0 = ret.Get(0).(bool)
	}
	return r0
}

// mockfailure_Is_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Is'
type mockfailure_Is_Call struct {
	*mock.Call
}

// Is is a helper method to define mock.On call
//   - target error
func (_e *mockfailure_Expecter) Is(target interface{}) *mockfailure_Is_Call {
	return &mockfailure_Is_Call{Call: _e.mock.On("Is", target)}
}

func (_c *mockfailure_Is_Call) Run(run func(target error)) *mockfailure_Is_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 error
		if args[0] != nil {
			arg0 = args[0].(error)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *mockfailure_Is_Call) Return(b bool) *mockfailure_Is_Call {
	_c.Call.Return(b)
	return _c
}

func (_c *mockfailure_Is_Call) RunAndReturn(run func(target error) bool) *mockfailure_Is_Call {
	_c.Call.Return(run)
	return _c
}

// Unwrap provides a mock function for the type mockfailure
func (_mock *mockfailure) Unwrap() error {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Unwrap")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func() error); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// mockfailure_Unwrap_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Unwrap'
type mockfailure_Unwrap_Call struct {
	*mock.Call
}

// Unwrap is a helper method to define mock.On call
func (_e *mockfailure_Expecter) Unwrap() *mockfailure_Unwrap_Call {
	return &mockfailure_Unwrap_Call{Call: _e.mock.On("Unwrap")}
}

func (_c *mockfailure_Unwrap_Call) Run(run func()) *mockfailure_Unwrap_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *mockfailure_Unwrap_Call) Return(err error) *mockfailure_Unwrap_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *mockfailure_Unwrap_Call) RunAndReturn(run func() error) *mockfailure_Unwrap_Call {
	_c.Call.Return(run)
	return _c
}
