// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/nicholas-fedor/shoutrrr/pkg/services/push/gotify"
	"github.com/nicholas-fedor/shoutrrr/pkg/types"
	mock "github.com/stretchr/testify/mock"
)

// NewMockPayloadBuilder creates a new instance of MockPayloadBuilder. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockPayloadBuilder(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockPayloadBuilder {
	mock := &MockPayloadBuilder{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockPayloadBuilder is an autogenerated mock type for the PayloadBuilder type
type MockPayloadBuilder struct {
	mock.Mock
}

type MockPayloadBuilder_Expecter struct {
	mock *mock.Mock
}

func (_m *MockPayloadBuilder) EXPECT() *MockPayloadBuilder_Expecter {
	return &MockPayloadBuilder_Expecter{mock: &_m.Mock}
}

// ParseExtras provides a mock function for the type MockPayloadBuilder
func (_mock *MockPayloadBuilder) ParseExtras(params *types.Params, config *gotify.Config) (map[string]any, error) {
	ret := _mock.Called(params, config)

	if len(ret) == 0 {
		panic("no return value specified for ParseExtras")
	}

	var r0 map[string]any
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(*types.Params, *gotify.Config) (map[string]any, error)); ok {
		return returnFunc(params, config)
	}
	if returnFunc, ok := ret.Get(0).(func(*types.Params, *gotify.Config) map[string]any); ok {
		r0 = returnFunc(params, config)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(map[string]any)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(*types.Params, *gotify.Config) error); ok {
		r1 = returnFunc(params, config)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockPayloadBuilder_ParseExtras_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'ParseExtras'
type MockPayloadBuilder_ParseExtras_Call struct {
	*mock.Call
}

// ParseExtras is a helper method to define mock.On call
//   - params *types.Params
//   - config *gotify.Config
func (_e *MockPayloadBuilder_Expecter) ParseExtras(params interface{}, config interface{}) *MockPayloadBuilder_ParseExtras_Call {
	return &MockPayloadBuilder_ParseExtras_Call{Call: _e.mock.On("ParseExtras", params, config)}
}

func (_c *MockPayloadBuilder_ParseExtras_Call) Run(run func(params *types.Params, config *gotify.Config)) *MockPayloadBuilder_ParseExtras_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *types.Params
		if args[0] != nil {
			arg0 = args[0].(*types.Params)
		}
		var arg1 *gotify.Config
		if args[1] != nil {
			arg1 = args[1].(*gotify.Config)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockPayloadBuilder_ParseExtras_Call) Return(stringToV map[string]any, err error) *MockPayloadBuilder_ParseExtras_Call {
	_c.Call.Return(stringToV, err)
	return _c
}

func (_c *MockPayloadBuilder_ParseExtras_Call) RunAndReturn(run func(params *types.Params, config *gotify.Config) (map[string]any, error)) *MockPayloadBuilder_ParseExtras_Call {
	_c.Call.Return(run)
	return _c
}

// PrepareRequest provides a mock function for the type MockPayloadBuilder
func (_mock *MockPayloadBuilder) PrepareRequest(message string, config *gotify.Config, extras map[string]any, date string) *gotify.MessageRequest {
	ret := _mock.Called(message, config, extras, date)

	if len(ret) == 0 {
		panic("no return value specified for PrepareRequest")
	}

	var r0 *gotify.MessageRequest
	if returnFunc, ok := ret.Get(0).(func(string, *gotify.Config, map[string]any, string) *gotify.MessageRequest); ok {
		r0 = returnFunc(message, config, extras, date)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*gotify.MessageRequest)
		}
	}
	return r0
}

// MockPayloadBuilder_PrepareRequest_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'PrepareRequest'
type MockPayloadBuilder_PrepareRequest_Call struct {
	*mock.Call
}

// PrepareRequest is a helper method to define mock.On call
//   - message string
//   - config *gotify.Config
//   - extras map[string]any
//   - date string
func (_e *MockPayloadBuilder_Expecter) PrepareRequest(message interface{}, config interface{}, extras interface{}, date interface{}) *MockPayloadBuilder_PrepareRequest_Call {
	return &MockPayloadBuilder_PrepareRequest_Call{Call: _e.mock.On("PrepareRequest", message, config, extras, date)}
}

func (_c *MockPayloadBuilder_PrepareRequest_Call) Run(run func(message string, config *gotify.Config, extras map[string]any, date string)) *MockPayloadBuilder_PrepareRequest_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 *gotify.Config
		if args[1] != nil {
			arg1 = args[1].(*gotify.Config)
		}
		var arg2 map[string]any
		if args[2] != nil {
			arg2 = args[2].(map[string]any)
		}
		var arg3 string
		if args[3] != nil {
			arg3 = args[3].(string)
		}
		run(
			arg0,
			arg1,
			arg2,
			arg3,
		)
	})
	return _c
}

func (_c *MockPayloadBuilder_PrepareRequest_Call) Return(messageRequest *gotify.MessageRequest) *MockPayloadBuilder_PrepareRequest_Call {
	_c.Call.Return(messageRequest)
	return _c
}

func (_c *MockPayloadBuilder_PrepareRequest_Call) RunAndReturn(run func(message string, config *gotify.Config, extras map[string]any, date string) *gotify.MessageRequest) *MockPayloadBuilder_PrepareRequest_Call {
	_c.Call.Return(run)
	return _c
}
