// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"net/url"

	"github.com/nicholas-fedor/shoutrrr/pkg/types"
	mock "github.com/stretchr/testify/mock"
)

// NewMockServiceConfig creates a new instance of MockServiceConfig. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockServiceConfig(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockServiceConfig {
	mock := &MockServiceConfig{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockServiceConfig is an autogenerated mock type for the ServiceConfig type
type MockServiceConfig struct {
	mock.Mock
}

type MockServiceConfig_Expecter struct {
	mock *mock.Mock
}

func (_m *MockServiceConfig) EXPECT() *MockServiceConfig_Expecter {
	return &MockServiceConfig_Expecter{mock: &_m.Mock}
}

// Enums provides a mock function for the type MockServiceConfig
func (_mock *MockServiceConfig) Enums() map[string]types.EnumFormatter {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Enums")
	}

	var r0 map[string]types.EnumFormatter
	if returnFunc, ok := ret.Get(0).(func() map[string]types.EnumFormatter); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(map[string]types.EnumFormatter)
		}
	}
	return r0
}

// MockServiceConfig_Enums_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Enums'
type MockServiceConfig_Enums_Call struct {
	*mock.Call
}

// Enums is a helper method to define mock.On call
func (_e *MockServiceConfig_Expecter) Enums() *MockServiceConfig_Enums_Call {
	return &MockServiceConfig_Enums_Call{Call: _e.mock.On("Enums")}
}

func (_c *MockServiceConfig_Enums_Call) Run(run func()) *MockServiceConfig_Enums_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockServiceConfig_Enums_Call) Return(stringToEnumFormatter map[string]types.EnumFormatter) *MockServiceConfig_Enums_Call {
	_c.Call.Return(stringToEnumFormatter)
	return _c
}

func (_c *MockServiceConfig_Enums_Call) RunAndReturn(run func() map[string]types.EnumFormatter) *MockServiceConfig_Enums_Call {
	_c.Call.Return(run)
	return _c
}

// GetURL provides a mock function for the type MockServiceConfig
func (_mock *MockServiceConfig) GetURL() *url.URL {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for GetURL")
	}

	var r0 *url.URL
	if returnFunc, ok := ret.Get(0).(func() *url.URL); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*url.URL)
		}
	}
	return r0
}

// MockServiceConfig_GetURL_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetURL'
type MockServiceConfig_GetURL_Call struct {
	*mock.Call
}

// GetURL is a helper method to define mock.On call
func (_e *MockServiceConfig_Expecter) GetURL() *MockServiceConfig_GetURL_Call {
	return &MockServiceConfig_GetURL_Call{Call: _e.mock.On("GetURL")}
}

func (_c *MockServiceConfig_GetURL_Call) Run(run func()) *MockServiceConfig_GetURL_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockServiceConfig_GetURL_Call) Return(uRL *url.URL) *MockServiceConfig_GetURL_Call {
	_c.Call.Return(uRL)
	return _c
}

func (_c *MockServiceConfig_GetURL_Call) RunAndReturn(run func() *url.URL) *MockServiceConfig_GetURL_Call {
	_c.Call.Return(run)
	return _c
}

// SetURL provides a mock function for the type MockServiceConfig
func (_mock *MockServiceConfig) SetURL(serviceURL *url.URL) error {
	ret := _mock.Called(serviceURL)

	if len(ret) == 0 {
		panic("no return value specified for SetURL")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(*url.URL) error); ok {
		r0 = returnFunc(serviceURL)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockServiceConfig_SetURL_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'SetURL'
type MockServiceConfig_SetURL_Call struct {
	*mock.Call
}

// SetURL is a helper method to define mock.On call
//   - serviceURL *url.URL
func (_e *MockServiceConfig_Expecter) SetURL(serviceURL interface{}) *MockServiceConfig_SetURL_Call {
	return &MockServiceConfig_SetURL_Call{Call: _e.mock.On("SetURL", serviceURL)}
}

func (_c *MockServiceConfig_SetURL_Call) Run(run func(serviceURL *url.URL)) *MockServiceConfig_SetURL_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *url.URL
		if args[0] != nil {
			arg0 = args[0].(*url.URL)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockServiceConfig_SetURL_Call) Return(err error) *MockServiceConfig_SetURL_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockServiceConfig_SetURL_Call) RunAndReturn(run func(serviceURL *url.URL) error) *MockServiceConfig_SetURL_Call {
	_c.Call.Return(run)
	return _c
}
