// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"net/http"

	mock "github.com/stretchr/testify/mock"
)

// NewMockClient creates a new instance of MockClient. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockClient(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockClient {
	mock := &MockClient{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockClient is an autogenerated mock type for the Client type
type MockClient struct {
	mock.Mock
}

type MockClient_Expecter struct {
	mock *mock.Mock
}

func (_m *MockClient) EXPECT() *MockClient_Expecter {
	return &MockClient_Expecter{mock: &_m.Mock}
}

// ErrorResponse provides a mock function for the type MockClient
func (_mock *MockClient) ErrorResponse(err error, response any) bool {
	ret := _mock.Called(err, response)

	if len(ret) == 0 {
		panic("no return value specified for ErrorResponse")
	}

	var r0 bool
	if returnFunc, ok := ret.Get(0).(func(error, any) bool); ok {
		r0 = returnFunc(err, response)
	} else {
		r0 = ret.Get(0).(bool)
	}
	return r0
}

// MockClient_ErrorResponse_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'ErrorResponse'
type MockClient_ErrorResponse_Call struct {
	*mock.Call
}

// ErrorResponse is a helper method to define mock.On call
//   - err error
//   - response any
func (_e *MockClient_Expecter) ErrorResponse(err interface{}, response interface{}) *MockClient_ErrorResponse_Call {
	return &MockClient_ErrorResponse_Call{Call: _e.mock.On("ErrorResponse", err, response)}
}

func (_c *MockClient_ErrorResponse_Call) Run(run func(err error, response any)) *MockClient_ErrorResponse_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 error
		if args[0] != nil {
			arg0 = args[0].(error)
		}
		var arg1 any
		if args[1] != nil {
			arg1 = args[1].(any)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockClient_ErrorResponse_Call) Return(b bool) *MockClient_ErrorResponse_Call {
	_c.Call.Return(b)
	return _c
}

func (_c *MockClient_ErrorResponse_Call) RunAndReturn(run func(err error, response any) bool) *MockClient_ErrorResponse_Call {
	_c.Call.Return(run)
	return _c
}

// Get provides a mock function for the type MockClient
func (_mock *MockClient) Get(url string, response any) error {
	ret := _mock.Called(url, response)

	if len(ret) == 0 {
		panic("no return value specified for Get")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, any) error); ok {
		r0 = returnFunc(url, response)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockClient_Get_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Get'
type MockClient_Get_Call struct {
	*mock.Call
}

// Get is a helper method to define mock.On call
//   - url string
//   - response any
func (_e *MockClient_Expecter) Get(url interface{}, response interface{}) *MockClient_Get_Call {
	return &MockClient_Get_Call{Call: _e.mock.On("Get", url, response)}
}

func (_c *MockClient_Get_Call) Run(run func(url string, response any)) *MockClient_Get_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 any
		if args[1] != nil {
			arg1 = args[1].(any)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockClient_Get_Call) Return(err error) *MockClient_Get_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockClient_Get_Call) RunAndReturn(run func(url string, response any) error) *MockClient_Get_Call {
	_c.Call.Return(run)
	return _c
}

// Headers provides a mock function for the type MockClient
func (_mock *MockClient) Headers() http.Header {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Headers")
	}

	var r0 http.Header
	if returnFunc, ok := ret.Get(0).(func() http.Header); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(http.Header)
		}
	}
	return r0
}

// MockClient_Headers_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Headers'
type MockClient_Headers_Call struct {
	*mock.Call
}

// Headers is a helper method to define mock.On call
func (_e *MockClient_Expecter) Headers() *MockClient_Headers_Call {
	return &MockClient_Headers_Call{Call: _e.mock.On("Headers")}
}

func (_c *MockClient_Headers_Call) Run(run func()) *MockClient_Headers_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockClient_Headers_Call) Return(header http.Header) *MockClient_Headers_Call {
	_c.Call.Return(header)
	return _c
}

func (_c *MockClient_Headers_Call) RunAndReturn(run func() http.Header) *MockClient_Headers_Call {
	_c.Call.Return(run)
	return _c
}

// Post provides a mock function for the type MockClient
func (_mock *MockClient) Post(url string, request any, response any) error {
	ret := _mock.Called(url, request, response)

	if len(ret) == 0 {
		panic("no return value specified for Post")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, any, any) error); ok {
		r0 = returnFunc(url, request, response)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockClient_Post_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Post'
type MockClient_Post_Call struct {
	*mock.Call
}

// Post is a helper method to define mock.On call
//   - url string
//   - request any
//   - response any
func (_e *MockClient_Expecter) Post(url interface{}, request interface{}, response interface{}) *MockClient_Post_Call {
	return &MockClient_Post_Call{Call: _e.mock.On("Post", url, request, response)}
}

func (_c *MockClient_Post_Call) Run(run func(url string, request any, response any)) *MockClient_Post_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 any
		if args[1] != nil {
			arg1 = args[1].(any)
		}
		var arg2 any
		if args[2] != nil {
			arg2 = args[2].(any)
		}
		run(
			arg0,
			arg1,
			arg2,
		)
	})
	return _c
}

func (_c *MockClient_Post_Call) Return(err error) *MockClient_Post_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockClient_Post_Call) RunAndReturn(run func(url string, request any, response any) error) *MockClient_Post_Call {
	_c.Call.Return(run)
	return _c
}
