// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"log"

	mock "github.com/stretchr/testify/mock"
)

// NewMockServiceOpts creates a new instance of MockServiceOpts. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockServiceOpts(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockServiceOpts {
	mock := &MockServiceOpts{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockServiceOpts is an autogenerated mock type for the ServiceOpts type
type MockServiceOpts struct {
	mock.Mock
}

type MockServiceOpts_Expecter struct {
	mock *mock.Mock
}

func (_m *MockServiceOpts) EXPECT() *MockServiceOpts_Expecter {
	return &MockServiceOpts_Expecter{mock: &_m.Mock}
}

// Logger provides a mock function for the type MockServiceOpts
func (_mock *MockServiceOpts) Logger() *log.Logger {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Logger")
	}

	var r0 *log.Logger
	if returnFunc, ok := ret.Get(0).(func() *log.Logger); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*log.Logger)
		}
	}
	return r0
}

// MockServiceOpts_Logger_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Logger'
type MockServiceOpts_Logger_Call struct {
	*mock.Call
}

// Logger is a helper method to define mock.On call
func (_e *MockServiceOpts_Expecter) Logger() *MockServiceOpts_Logger_Call {
	return &MockServiceOpts_Logger_Call{Call: _e.mock.On("Logger")}
}

func (_c *MockServiceOpts_Logger_Call) Run(run func()) *MockServiceOpts_Logger_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockServiceOpts_Logger_Call) Return(logger *log.Logger) *MockServiceOpts_Logger_Call {
	_c.Call.Return(logger)
	return _c
}

func (_c *MockServiceOpts_Logger_Call) RunAndReturn(run func() *log.Logger) *MockServiceOpts_Logger_Call {
	_c.Call.Return(run)
	return _c
}

// Props provides a mock function for the type MockServiceOpts
func (_mock *MockServiceOpts) Props() map[string]string {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Props")
	}

	var r0 map[string]string
	if returnFunc, ok := ret.Get(0).(func() map[string]string); ok {
		r0 = returnFunc()
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(map[string]string)
		}
	}
	return r0
}

// MockServiceOpts_Props_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Props'
type MockServiceOpts_Props_Call struct {
	*mock.Call
}

// Props is a helper method to define mock.On call
func (_e *MockServiceOpts_Expecter) Props() *MockServiceOpts_Props_Call {
	return &MockServiceOpts_Props_Call{Call: _e.mock.On("Props")}
}

func (_c *MockServiceOpts_Props_Call) Run(run func()) *MockServiceOpts_Props_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockServiceOpts_Props_Call) Return(stringToString map[string]string) *MockServiceOpts_Props_Call {
	_c.Call.Return(stringToString)
	return _c
}

func (_c *MockServiceOpts_Props_Call) RunAndReturn(run func() map[string]string) *MockServiceOpts_Props_Call {
	_c.Call.Return(run)
	return _c
}

// Verbose provides a mock function for the type MockServiceOpts
func (_mock *MockServiceOpts) Verbose() bool {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Verbose")
	}

	var r0 bool
	if returnFunc, ok := ret.Get(0).(func() bool); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Get(0).(bool)
	}
	return r0
}

// MockServiceOpts_Verbose_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Verbose'
type MockServiceOpts_Verbose_Call struct {
	*mock.Call
}

// Verbose is a helper method to define mock.On call
func (_e *MockServiceOpts_Expecter) Verbose() *MockServiceOpts_Verbose_Call {
	return &MockServiceOpts_Verbose_Call{Call: _e.mock.On("Verbose")}
}

func (_c *MockServiceOpts_Verbose_Call) Run(run func()) *MockServiceOpts_Verbose_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockServiceOpts_Verbose_Call) Return(b bool) *MockServiceOpts_Verbose_Call {
	_c.Call.Return(b)
	return _c
}

func (_c *MockServiceOpts_Verbose_Call) RunAndReturn(run func() bool) *MockServiceOpts_Verbose_Call {
	_c.Call.Return(run)
	return _c
}
